<?php

namespace App\Http\Controllers;

use App\AdminOtherAccount;
use App\AdminTermsAndCondition;
use App\Company;
use App\CreditNote;
use App\Customer;
use App\Expense;
use App\OtherAccountBuyer;
use App\PurchaseOrder;
use App\StaffAndSalesman;
use App\Supplier;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Permission;
use App\Branches;

class PurchaseOrderController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */

    public function index()
    {
        // $purchaseOrders = PurchaseOrder::whereIn('id', function ($query) {
        //         $query->selectRaw('MIN(id)')
        //             ->from('purchase_orders')
        //             ->groupBy('invoice_no');
        //     })
        //     ->where('branch_id', session()->get('selected_branch'))
        //     ->withTrashed()
        //     ->orderBy('id', 'desc')->get();

        $purchaseOrders = PurchaseOrder::whereIn('id', function ($query) {
            $query->selectRaw('MIN(id)')
                ->from('purchase_orders')
                ->where('branch_id', session()->get('selected_branch'))
                ->whereNotNull('invoice_no')
                ->groupBy('invoice_no');
        })
        ->withTrashed()
        ->orderBy('invoice_no', 'desc')
        ->get();

        return view('crusher.trans_purchase_order.index', compact('purchaseOrders'));
    }


    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {

         // Retrieve permission titles from Permission table
         $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

         // Filter permission titles using hasBranchPermission function
         $filteredPermissionTitles = [];
         $branchId = session()->get('selected_branch');
         foreach ($permissionTitles as $permissionTitle) {
             if (Branches::hasBranchPermission($branchId, $permissionTitle)) {
                 $filteredPermissionTitles[] = $permissionTitle;
             }
         }


        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))->get();
        $currentDate = Carbon::now()->format('Ymd');

        // Get the latest quotation number for the branch
        $latestPo = PurchaseOrder::withTrashed()->where('branch_id', session()->get('selected_branch'))
            ->latest('invoice_no')->first();

        // Calculate the new quotation number
        if ($latestPo) {
            $lastPon = (int)substr($latestPo->invoice_no, -4); // Extract the last 4 digits
            $newNumber = $lastPon + 1;
            $newPon = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
        } else {
            $newPon = '0001';
        }



        // // Retrieve customers and include both 'account_name' and 'table_name' attributes
        // $customers = Customer::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Customer');
        // });

        // // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        // $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Supplier');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        // ->whereIn(DB::raw("CONCAT(first_name, ' ', last_name)"), $filteredPermissionTitles)
        // ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Expense');
        // });

        // // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        // $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        // });

        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("CONCAT(REPLACE(first_name, ' ', '-'), '-', REPLACE(last_name, ' ', '-'))"), $filteredPermissionTitles)
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) AS account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);


        return view('crusher.trans_purchase_order.create', compact('newPon', 'customers', 'accounts'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {

        $latestNo = PurchaseOrder::withTrashed()->where('branch_id', session()->get('selected_branch'))
        ->latest('invoice_no')->first();

        if ($latestNo) {
        $latestInvoiceNo = $latestNo->invoice_no;

        $latestNumber = (int)preg_replace('/[^0-9]/', '', $latestInvoiceNo);

        $newNumber = $latestNumber + 1;

        $newInvoiceNo = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
        } else {
        $newInvoiceNo = '0001';
        }
        try {
            $validatedData = $request->validate([
                // 'invoice_no' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable',
                'acc_no' => 'nullable',
                'account_type' => 'nullable',
                'account_name' => 'nullable',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'quantity' => 'nullable|array',
                'quantity.*' => 'nullable|numeric',
                'rate_unit' => 'nullable|array',
                'rate_unit.*' => 'nullable|numeric',
                'gross_amount' => 'nullable|array',
                'gross_amount.*' => 'nullable|numeric',
                'vat_percentage' => 'nullable|array',
                'vat_percentage.*' => 'nullable|numeric',
                'vat_amount' => 'nullable|array',
                'vat_amount.*' => 'nullable|numeric',
                'net_amount' => 'nullable|array',
                'net_amount.*' => 'nullable|numeric',
                'expense_id' => 'nullable|array',
                'expense_id.*' => 'nullable',
                'expense_amount' => 'nullable|array',
                'expense_amount.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'total_gross_amount' => 'nullable|numeric',
                'total_vat_amount' => 'nullable|numeric',
                'total_net_amount' => 'nullable|numeric',
                'name' => 'nullable|string',
                'phone' => 'nullable|string',
                'email' => 'nullable|string',
                'vat_number' => 'nullable|string',
                'address' => 'nullable|string',
                'type' => 'nullable|string',
            ]);
            $branch_id = session()->get('selected_branch');


            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['quantity']); $i++)
            {
                $quotationData = [
                    'invoice_no' => $newInvoiceNo,
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'acc_no' => $validatedData['acc_no'],
                    'account_type' => $validatedData['account_type'],
                    'account_name' => $validatedData['account_name'],
                    'item_description' => $validatedData['item_description'][$i]??'',
                    'quantity' => $validatedData['quantity'][$i]??"",
                    'rate_unit' => $validatedData['rate_unit'][$i]??'',
                    'gross_amount' => $validatedData['gross_amount'][$i]??"",
                    'vat_percentage' => $validatedData['vat_percentage'][$i]??'',
                    'vat_amount' => $validatedData['vat_amount'][$i]??'',
                    'net_amount' => $validatedData['net_amount'][$i]??'',
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'branch_id' => $branch_id,
                ];

                PurchaseOrder::create($quotationData);
            }

            flash()->addSuccess('Record Created');
            return redirect()->route('software.purchase_orders.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($purchaseOrder)
    {
        $purchaseOrder = PurchaseOrder::withTrashed()->find($purchaseOrder);
        $purchaseOrder = $purchaseOrder->withTrashed()->with('account')->where('invoice_no', $purchaseOrder->invoice_no)->where('branch_id', session()->get('selected_branch'))->get();
        $company = Company::where('user_id', auth()->id())->first();
        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'purchase_order'])->first();
        return view('crusher.trans_purchase_order.view', compact('purchaseOrder', 'company', 'tnc'));
    }

    public function edit(PurchaseOrder $purchaseOrder)
    {

        // Retrieve permission titles from Permission table
        $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

        // Filter permission titles using hasBranchPermission function
        $filteredPermissionTitles = [];
        $branchId = session()->get('selected_branch');
        foreach ($permissionTitles as $permissionTitle) {
            if (Branches::hasBranchPermission($branchId, $permissionTitle)) {
                $filteredPermissionTitles[] = $permissionTitle;
            }
        }

        $expenses = Expense::where('branch_id', session()->get('selected_branch'))->get();
        $currentDate = Carbon::now()->format('Ymd');

        // // Retrieve customers and include both 'account_name' and 'table_name' attributes
        // $customers = Customer::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Customer');
        // });

        // // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        // $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Supplier');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        // ->whereIn(DB::raw("CONCAT(first_name, ' ', last_name)"), $filteredPermissionTitles)
        // ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Expense');
        // });

        // // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        // $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        // });

        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("CONCAT(REPLACE(first_name, ' ', '-'), '-', REPLACE(last_name, ' ', '-'))"), $filteredPermissionTitles)
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) AS account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);
        // $purchaseOrders = $purchaseOrder->with('account')->where('invoice_no', $purchaseOrder->invoice_no)->get();

        $purchaseOrders = PurchaseOrder::where('branch_id', session()->get('selected_branch'))
        ->with('account')
        ->where('invoice_no', $purchaseOrder->invoice_no)
        ->get();


        return view('crusher.trans_purchase_order.edit', compact('accounts', 'purchaseOrders'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, PurchaseOrder $purchaseOrder)
    {
        // dd($request);
        try {
            $validatedData = $request->validate([
                'invoice_no' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable',
                'acc_no' => 'nullable',
                'account_type' => 'nullable',
                'account_name' => 'nullable',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'quantity' => 'nullable|array',
                'quantity.*' => 'nullable|numeric',
                'rate_unit' => 'nullable|array',
                'rate_unit.*' => 'nullable|numeric',
                'gross_amount' => 'nullable|array',
                'gross_amount.*' => 'nullable|numeric',
                'vat_percentage' => 'nullable|array',
                'vat_percentage.*' => 'nullable|numeric',
                'vat_amount' => 'nullable|array',
                'vat_amount.*' => 'nullable|numeric',
                'net_amount' => 'nullable|array',
                'net_amount.*' => 'nullable|numeric',
                'expense_id' => 'nullable|array',
                'expense_id.*' => 'nullable',
                'expense_amount' => 'nullable|array',
                'expense_amount.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'total_gross_amount' => 'nullable|numeric',
                'total_vat_amount' => 'nullable|numeric',
                'total_net_amount' => 'nullable|numeric',
            ]);
            $branch_id = session()->get('selected_branch');

            $purchaseOrder = $purchaseOrder->where('branch_id', session()->get('selected_branch'))->with('account')->where('invoice_no', $purchaseOrder->invoice_no)->forceDelete();

            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['quantity']); $i++)
            {
                $quotationData = [
                    'invoice_no' => $validatedData['invoice_no'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'acc_no' => $validatedData['acc_no'],
                    'account_type' => $validatedData['account_type'],
                    'account_name' => $validatedData['account_name'],
                    'item_description' => $validatedData['item_description'][$i]??'',
                    'quantity' => $validatedData['quantity'][$i]??"",
                    'rate_unit' => $validatedData['rate_unit'][$i]??'',
                    'gross_amount' => $validatedData['gross_amount'][$i]??"",
                    'vat_percentage' => $validatedData['vat_percentage'][$i]??'',
                    'vat_amount' => $validatedData['vat_amount'][$i]??'',
                    'net_amount' => $validatedData['net_amount'][$i]??'',
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'branch_id' => $branch_id,
                ];

                PurchaseOrder::create($quotationData);
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('software.purchase_orders.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */

    public function destroy(Quotation $quotation)
    {
        //
    }

    public function softDelete($purchaseOrder)
    {
        $purchaseOrder = PurchaseOrder::findOrFail($purchaseOrder);
        $purchaseOrder->where('branch_id', session()->get('selected_branch'))->where('invoice_no', $purchaseOrder->invoice_no)->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.purchase_orders.index');
    }

    function buyer(Request $request)
    {
        // return $request->all();
        try {
            unset($request['buyer_account_id']);
            unset($request['_token']);
            OtherAccountBuyer::firstOrCreate(['type'=>$request->type, 'invoice_no'=>$request->invoice_no],$request->all());
            return response()->json(['status'=>true]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }

    function buyerExists(Request $request)
    {
        // return $request->all();
        try {
            $buyer = OtherAccountBuyer::where(['type'=>$request->type, 'invoice_no'=>$request->invoice_no])->exists();
            return response()->json(['status'=>$buyer]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }

    function buyerDelete(Request $request)
    {
        // return $request->all();
        try {
            $buyer = OtherAccountBuyer::where(['type'=>$request->type, 'invoice_no'=>$request->invoice_no])->delete();
            return response()->json(['status'=>$buyer]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }
}
