<?php

namespace App\Http\Controllers\Report;

use App\Currency;
use App\Http\Controllers\Controller;
use App\PurchaseInvoice;
use App\PurchaseReturnVoucher;
use App\SaleInvoice;
use App\SalesReturnVoucher;
use App\Services\Utils;
use PDF;
use Auth;
use Illuminate\Http\Request;

class DayBookController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function display()
    {
        $user = Auth::user();
        if (!$user->can('day-book-report')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.reports.day_book.select_date');
    }

    public function index(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('day-book-report')) {
            abort(403, 'Unauthorized action.');
        }

        // dd($request->all());
        $location_id = Utils::getLocation();
        $from_date = $request->from_date;
        $to_date = $request->to_date;

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $sale_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($sale_invoices as $key => $sale_invoice) {
            $sale_invoice-> invoice_type = 'sales';
        }
        $purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('invoice_no', '!=' , '0')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($purchase_invoices as $key => $purchase_invoice) {
            $purchase_invoice-> invoice_type = 'purchase';
        }
        $purchase_return = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($purchase_return as $key => $_purchase_return) {
            $_purchase_return-> invoice_type = 'purchase return';
        }
        $sale_return = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($sale_return as $key => $_sale_return) {
            $_sale_return-> invoice_type = 'sale return';
        }

        $unsorted_daybook_data  = $sale_invoices->concat($purchase_invoices)->concat($purchase_return)->concat($sale_return);
        $collection = collect($unsorted_daybook_data);
        $sorted_daybook_data = $collection->sortBy('date')->values()->all();
        // return $sorted_daybook_data;
        return view('user.reports.day_book.index', compact('decimal','from_date','to_date','sorted_daybook_data'));
    }

    public function pdfView($from_date, $to_date){

        $user = Auth::user();
        if (!$user->can('day-book-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $sale_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($sale_invoices as $key => $sale_invoice) {
            $sale_invoice-> invoice_type = 'sales';
        }
        $purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('invoice_no', '!=' , '0')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($purchase_invoices as $key => $purchase_invoice) {
            $purchase_invoice-> invoice_type = 'purchase';
        }
        $purchase_return = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($purchase_return as $key => $_purchase_return) {
            $_purchase_return-> invoice_type = 'purchase return';
        }
        $sale_return = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($sale_return as $key => $_sale_return) {
            $_sale_return-> invoice_type = 'sale return';
        }

        $unsorted_daybook_data  = $sale_invoices->concat($purchase_invoices)->concat($purchase_return)->concat($sale_return);
        $collection = collect($unsorted_daybook_data);
        $sorted_daybook_data = $collection->sortBy('date')->values()->all();


        $pdf = PDF::loadView('user.reports.day_book.list', compact('from_date','to_date','sorted_daybook_data', 'decimal'));
        return $pdf->stream();


    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
