<?php

namespace App\Http\Controllers\Report;

use App\AccountMaster;
use App\Http\Controllers\Controller;
use App\ClassOfAccount;
use App\AccountMastersAccountsDetail;
use App\Currency;
use App\Services\Utils;
use Illuminate\Http\Request;
use PDF;
use Auth;

class TrialBalanceReportController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('trial-balance-report')) {
            abort(403, 'Unauthorized action.');
        }

        // dd($request->all());
        $location_id = Utils::getLocation();
        $from_date = $request->from_date;
        $to_date = $request->to_date;

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $class_of_accounts = ClassOfAccount::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->get();
        foreach ($class_of_accounts as $class_of_account){
            $debit = 0;
            $credit = 0;
            foreach ($class_of_account->account_master as $account_master){
                $debit = $debit + AccountMastersAccountsDetail::with(['account_master' => function($q) use ($location_id) {
                    $q->where('location_id', $location_id);}])->Where('account_master_id', $account_master->id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('debit');
                $credit = $credit + AccountMastersAccountsDetail::with(['account_master' => function($q) use ($location_id) {
                    $q->where('location_id', $location_id);}])->Where('account_master_id', $account_master->id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('credit');
            }
            $trial_balance_debit_credit[] = [
                "class_of_account" => $class_of_account->name,
                "debit" => $debit,
                "debit_decimal" => number_format($debit, $decimal, '.', ''),
                "credit" => $credit,
                "credit_decimal" => number_format($credit, $decimal, '.', '')
            ];
        }
        // return $trial_balance_debit_credit;

        return view('user.reports.trial_balance_report.index', compact('from_date','to_date', 'trial_balance_debit_credit'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function display()
    {
        $user = Auth::user();
        if (!$user->can('trial-balance-report')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.reports.trial_balance_report.select_date');
    }

    public function pdfView($from_date, $to_date){

        $user = Auth::user();
        if (!$user->can('trial-balance-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $class_of_accounts = ClassOfAccount::with('account_master')->get();
        foreach ($class_of_accounts as $class_of_account){
            $debit = 0;
            $credit = 0;
            foreach ($class_of_account->account_master as $account_master){
                $debit = $debit + AccountMastersAccountsDetail::with(['account_master' => function($q) use ($location_id) {
                    $q->where('location_id', $location_id);}])->Where('account_master_id', $account_master->id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('debit');
                $credit = $credit + AccountMastersAccountsDetail::with(['account_master' => function($q) use ($location_id) {
                    $q->where('location_id', $location_id);}])->Where('account_master_id', $account_master->id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('credit');
            }
            $trial_balance_debit_credit[] = [
                "class_of_account" => $class_of_account->name,
                "debit" => $debit,
                "credit" => $credit,
                "debit_decimal" => number_format($debit, $decimal, '.', ''),
                "credit_decimal" => number_format($credit, $decimal, '.', '')
            ];
        }
        // return $trial_balance_debit_credit;

        // return view('user.reports.trial_balance_report.index', compact('from_date','to_date', 'trial_balance_debit_credit'));
        $pdf = PDF::loadView('user.reports.trial_balance_report.list', compact('from_date','to_date', 'trial_balance_debit_credit'));
        return $pdf->stream();

    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
