<?php

namespace App\Http\Controllers\Master\TermsAndConditions;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
use App\TermsAndConditionsSalesReturnVoucher;

class SalesReturnVoucherController extends Controller
{

    protected $utils;

    public function __construct(Utils $utils)
    {
        $this->utils = $utils;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('terms-and-conditions')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $terms_and_conditions = TermsAndConditionsSalesReturnVoucher::Where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(25);
        return view('user.masters.terms_and_conditions.sales_return_voucher.index', compact('terms_and_conditions'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('terms-and-conditions')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.masters.terms_and_conditions.sales_return_voucher.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $location_id = $this->getLocationId();
        $terms_and_conditions = new TermsAndConditionsSalesReturnVoucher();
        $terms_and_conditions->location_id = $location_id;
        $terms_and_conditions->terms_and_conditions = $request->terms_and_conditions;
        $terms_and_conditions->save();
        Session::flash('alert-success', 'Terms And Conditions created successfully');
        return redirect('/masters/terms-and-conditions/sales-return-voucher');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\TermsAndConditionsSalesReturnVoucher  $terms_and_conditions
     * @return \Illuminate\Http\Response
     */
    public function show(TermsAndConditionsSalesReturnVoucher $terms_and_conditions)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\TermsAndConditionsSalesReturnVoucher  $terms_and_conditions
     * @return \Illuminate\Http\Response
     */
    public function edit($terms_and_conditions_id)
    {
        $user = Auth::user();
        if (!$user->can('terms-and-conditions')) {
            abort(403, 'Unauthorized action.');
        }

        $terms_and_conditions = TermsAndConditionsSalesReturnVoucher::find($terms_and_conditions_id);
        return view('user.masters.terms_and_conditions.sales_return_voucher.edit', compact('terms_and_conditions'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\TermsAndConditionsSalesReturnVoucher  $terms_and_conditions
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $terms_and_conditions_id)
    {

        $terms_and_conditions = TermsAndConditionsSalesReturnVoucher::find($terms_and_conditions_id);
        $terms_and_conditions->terms_and_conditions = $request->terms_and_conditions;
        $terms_and_conditions->save();
        Session::flash('alert-success', 'Terms And Conditions updated successfully');
        return redirect('/masters/terms-and-conditions/sales-return-voucher');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\TermsAndConditionsSalesReturnVoucher  $terms_and_conditions
     * @return \Illuminate\Http\Response
     */
    public function destroy($terms_and_conditions_id)
    {
        $terms_and_conditions = TermsAndConditionsSalesReturnVoucher::findOrFail($terms_and_conditions_id);
        $terms_and_conditions->delete();
        Session::flash('alert-success', 'Terms And Conditions deleted successfully');
        return redirect('/masters/terms-and-conditions/sales-return-voucher');
    }
}
