<?php

namespace App\Http\Controllers\Transaction;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;


use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
use App\Services\Slug;
use App\WareHouse;
use App\ItemName;
use App\SaleInvoice;
use App\PurchaseInvoice;
use App\SaleInvoiceItem;
use App\StaffAndSalesman;
use App\Supplier;
use App\AccountMasterTrack;
use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use App\ApprovalVoucher;
use App\TermsAndConditionsSalesInvoice;
use App\Company;
use App\Currency;
use App\PurchaseSalesTransferStockTrack;
use App\WareHouseTrack;

class SaleInvoiceController extends Controller
{
    protected $utils;
    protected $slug;

    public function __construct(Utils $utils, Slug $slug)
    {
        $this->utils = $utils;
        $this->slug = $slug;
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('sales-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $sales_invoices = SaleInvoice::withTrashed()->with('account')->where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(100);
        return view('user.transactions.sales_invoices.index', compact('sales_invoices'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('sales-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $latest_invoice_vat = SaleInvoice::where('location_id', $location_id)->Where('vat_check', 1)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
        if($latest_invoice_vat){
            $invoice_no_vat = $latest_invoice_vat-> invoice_no + 1;
        }else{
            $invoice_no_vat = 1;
        }

        $latest_invoice = SaleInvoice::where('location_id', $location_id)->Where('vat_check', 0)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
        if($latest_invoice){
            $invoice_no = (int)$latest_invoice-> invoice_no + 1 .'A' ;

        }else{
            $invoice_no = '1A';
        }

        $salesmen = AccountMaster::where('account_master_type', "Staff-And-Salesman")->where('job_designation', 'salesman')->get();

        $tax_accounts = AccountMaster::where('class_of_account_id', 18)->get();

        $warehouses = WareHouseTrack::with(['warehouse' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $item_names = ItemName::all();

        $items = PurchaseSalesTransferStockTrack::Where('location_id', $location_id)->Where('quantity', '!=', 0)->get();
        $currency = Currency::Where('location_id', $location_id)->first();
        $terms_and_conditions_sales_invoice = TermsAndConditionsSalesInvoice::where('location_id', $location_id)->first();
        return view('user.transactions.sales_invoices.create', compact('currency','invoice_no_vat','invoice_no', 'account_masters', 'salesmen', 'tax_accounts', 'warehouses','item_names', 'items', 'terms_and_conditions_sales_invoice'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request->all());
        DB::beginTransaction();

        try {
            if ($request->approval_voucher != "") {
                $approval_voucher =  ApprovalVoucher::Find($request->approval_voucher);
            }
            $location_id = $this->getLocationId();
            $sales_invoice = new SaleInvoice();
            $sales_invoice->location_id = $location_id;
            $sales_invoice->invoice_no = $request->invoice_no;
            $sales_invoice->mode_of_payment = $request->mode_of_payment;
            $sales_invoice->date = $request->date;
            $sales_invoice->purchase_order = $request->purchase_order;
            $sales_invoice->lpo_days = $request->lpo_days;
            if ($request->vat_check == "") {
                $sales_invoice->vat_check = 0;
            }else {
                $sales_invoice->vat_check = $request->vat_check;
            }
            if ($request->approval_voucher != "") {
            $sales_invoice->delivered_order_number = $approval_voucher->invoice_no;
            }
            else{
            $sales_invoice->delivered_order_number = "";
            }
            $sales_invoice->salesman_id = $request->salesman_id;
            // $sales_invoice->warehouse_id = $request->warehouse_id;
            $sales_invoice->account_id = $request->account_id;
            $sales_invoice->total_quantity = $request->total_quantity;
            $sales_invoice->gross_total = $request->gross_total;
            $sales_invoice->discount = $request->discount_amount;
            $sales_invoice->tax_account_id= $request->tax_account_id;
            $sales_invoice->vat_percentage= $request->vat_percentage;
            $sales_invoice->vat_amount= $request->vat_amount;
            $sales_invoice->net_total = $request->net_total;
            $sales_invoice->paid_amount = 0;
            $sales_invoice->remaining_amount = $request->net_total;
            $sales_invoice->amount_in_words = $request->amount_in_words;
            $sales_invoice->customer_id = 0;
            $sales_invoice->narrations = $request->narrations;
            $sales_invoice->buyer_name = $request->buyer_name;
            $sales_invoice->buyer_mobile = $request->buyer_mobile;
            $sales_invoice->buyer_email = $request->buyer_email;
            $sales_invoice->buyer_address = $request->buyer_address;
            $sales_invoice->buyer_vat_number = $request->buyer_vat_number;
            $sales_invoice->is_signed = 0;
            $sales_invoice->save();

            foreach ($request->imei_serial_no as $key => $imei_serial_no) {
                $sales_invoice_item = new SaleInvoiceItem();
                $sales_invoice_item-> sales_invoice_id = $sales_invoice-> id;
                $sales_invoice_item-> item_name_id = $request->item_name_id[$key];
                $sales_invoice_item-> item_unit_id = $request->item_unit[$key];
                $sales_invoice_item-> imei_serial_no = $request->imei_serial_no[$key];
                $sales_invoice_item-> warehouse_id = $request->warehouse_id[$key];
                $sales_invoice_item-> quantity = $request->quantity[$key];
                $sales_invoice_item-> rate_per_quantity = $request->rate_per_quantity[$key];
                $sales_invoice_item-> amount = $request->amount[$key];
                $sales_invoice_item-> remarks = $request->remarks[$key];
                $sales_invoice_item-> save();

                $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $request->imei_serial_no[$key])->Where('location_id', $location_id)->first();
                if ($purchase_sales_transfer_stock_track) {
                    $purchase_sales_transfer_stock_track->quantity = $purchase_sales_transfer_stock_track->quantity - $request->quantity[$key];
                    $purchase_sales_transfer_stock_track->save();
                }else{
                    return redirect()->back()->withErrors("IMEI Not Found In This Location");
                }
            }
            if ($request->approval_voucher != "") {
                // $approval_voucher =  ApprovalVoucher::Find($request->approval_voucher);
                $approval_voucher->sales_invoice_status = 1;
                $approval_voucher->sales_invoice_id = $sales_invoice-> id;
                $approval_voucher->save();
            }

            //for accounts
            $account = new AccountMastersAccountsDetail();
            $account->account_master_id = $request->account_id;
            $account->credit = 0;
            $account->debit = $request->net_total;
            $account->date = $request->date;
            $account->voucher_type = "sale";
            $account->voucher_no = $request->invoice_no;
            $account->save();

            //for tax account
            $tax_account = new AccountMastersAccountsDetail();
            $tax_account->account_master_id = $request->tax_account_id;
            $tax_account->credit = $request->vat_amount;
            $tax_account->debit = 0;
            $tax_account->date = $request->date;
            $tax_account->voucher_type = "sale";
            $tax_account->voucher_no = $request->invoice_no;
            $tax_account->save();

            DB::commit();
            Session::flash('alert-success','Sales Invoice created successfully');
            return redirect('/transactions/sales-invoices');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\SaleInvoice  $saleInvoice
     * @return \Illuminate\Http\Response
     */
    public function show(SaleInvoice $saleInvoice)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\SaleInvoice  $saleInvoice
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = Auth::user();
        if (!$user->can('edit-sales-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $sales_invoice = SaleInvoice::with('items')->where('id', $id)->first();

        if ($sales_invoice->vat_check == 1) {

            $invoice_no_vat = $sales_invoice->invoice_no;

            $latest_invoice = SaleInvoice::where('location_id', $location_id)->Where('vat_check', 0)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
            if($latest_invoice){
                $invoice_no = (int)$latest_invoice-> invoice_no + 1 .'A' ;

            }else{
                $invoice_no = '1A';
            }

        }elseif ($sales_invoice->vat_check == 0) {

            $invoice_no = $sales_invoice->invoice_no;

            $latest_invoice_vat = SaleInvoice::where('location_id', $location_id)->Where('vat_check', 1)->withTrashed()->orderByRaw('CONVERT(invoice_no, SIGNED) desc')->first();
            if($latest_invoice_vat){
                $invoice_no_vat = $latest_invoice_vat-> invoice_no + 1;
            }else{
                $invoice_no_vat = 1;
            }
        }

        $salesmen = AccountMaster::Where('account_master_type', 'Staff-And-Salesman')->where('job_designation', 'salesman')->get();

        $tax_accounts = AccountMaster::where('class_of_account_id', 18)->get();

        $warehouses = WareHouseTrack::with(['warehouse' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $item_names = ItemName::all();

        // $items = PurchaseSalesTransferStockTrack::Where('location_id', $location_id)->Where('quantity', '!=', 0)->get();
        $items = PurchaseSalesTransferStockTrack::Where('location_id', $location_id)->get();

        $account_id = $sales_invoice->account_id;
        // $pass = AccountMaster::Where('id', $account)->pluck('transactions_password');
        $account = AccountMaster::Find($account_id);

        if ($account->transactions_password != "") {
            $pass_check = "yes";
        }elseif ($account->transactions_password == "") {
            $pass_check = "no";
        }
        $currency = Currency::Where('location_id', $location_id)->first();
// dd($pass_check);
        $terms_and_conditions_sales_invoice = TermsAndConditionsSalesInvoice::where('location_id', $location_id)->first();
        return view('user.transactions.sales_invoices.edit', compact('invoice_no','invoice_no_vat','currency','sales_invoice', 'account_masters', 'salesmen', 'tax_accounts', 'warehouses','item_names', 'items', 'terms_and_conditions_sales_invoice', 'pass_check', 'account'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\SaleInvoice  $saleInvoice
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request,$id)
    {
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $sales_invoice = SaleInvoice::where('id', $id)->first();
            $previous_net_total = (float)$sales_invoice->net_total;
            $current_net_total = (float)$request->net_total;
            $account = AccountMastersAccountsDetail::Where('account_master_id', $sales_invoice->account_id)->Where('voucher_type', "sale")->Where('voucher_no', $sales_invoice->invoice_no)->first();
            $sales_invoice->location_id = $location_id;
            $sales_invoice->invoice_no = $request->invoice_no;
            $sales_invoice->mode_of_payment = $request->mode_of_payment;
            $sales_invoice->date = $request->date;
            $sales_invoice->purchase_order = $request->purchase_order;
            $sales_invoice->lpo_days = $request->lpo_days;
            if ($request->vat_check == "") {
                $sales_invoice->vat_check = 0;
            }else {
                $sales_invoice->vat_check = $request->vat_check;
            }

            // $sales_invoice->delivered_order_number = "";
            $sales_invoice->salesman_id = $request->salesman_id;
            // $sales_invoice->warehouse_id = $request->warehouse_id;
            $sales_invoice->account_id = $request->account_id;
            $sales_invoice->total_quantity = $request->total_quantity;
            $sales_invoice->gross_total = $request->gross_total;
            $sales_invoice->discount = $request->discount_amount;
            $sales_invoice->tax_account_id= $request->tax_account_id;
            $sales_invoice->vat_percentage= $request->vat_percentage;
            $sales_invoice->vat_amount= $request->vat_amount;
            $sales_invoice->net_total = $request->net_total;
            $sales_invoice->remaining_amount = $request->net_total;
            $sales_invoice->amount_in_words = $request->amount_in_words;
            $sales_invoice->customer_id = 0;
            $sales_invoice->narrations = $request->narrations;
            $sales_invoice->buyer_name = $request->buyer_name;
            $sales_invoice->buyer_mobile = $request->buyer_mobile;
            $sales_invoice->buyer_email = $request->buyer_email;
            $sales_invoice->buyer_address = $request->buyer_address;
            $sales_invoice->buyer_vat_number = $request->buyer_vat_number;
            $sales_invoice->save();

            // SaleInvoiceItem::where('sales_invoice_id', $id)->forcedelete();
            $sale_items = SaleInvoiceItem::where('sales_invoice_id', $id)->get();
            foreach ($sale_items as $item) {
                $purchase_sale_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $item->imei_serial_no)->first();
                $quantity = $item->quantity;
                $purchase_sale_track->quantity = $purchase_sale_track->quantity + $quantity;
                $purchase_sale_track->save();

                $item->forcedelete();
            }

            foreach ($request->imei_serial_no as $key => $imei_serial_no) {


                $sales_invoice_item = new SaleInvoiceItem();
                $sales_invoice_item-> sales_invoice_id = $sales_invoice-> id;
                $sales_invoice_item-> item_name_id = $request->item_name_id[$key];
                $sales_invoice_item-> item_unit_id = $request->item_unit[$key];
                $sales_invoice_item-> imei_serial_no = $request->imei_serial_no[$key];
                $sales_invoice_item-> warehouse_id = $request->warehouse_id[$key];
                $sales_invoice_item-> quantity = $request->quantity[$key];
                $sales_invoice_item-> rate_per_quantity = $request->rate_per_quantity[$key];
                $sales_invoice_item-> amount = $request->amount[$key];
                $sales_invoice_item-> remarks = $request->remarks[$key];
                $sales_invoice_item-> save();

                $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $request->imei_serial_no[$key])->Where('location_id', $location_id)->first();
                if ($purchase_sales_transfer_stock_track) {
                    $purchase_sales_transfer_stock_track->quantity = $purchase_sales_transfer_stock_track->quantity - $request->quantity[$key];
                    $purchase_sales_transfer_stock_track->save();
                }else{
                    return redirect()->back()->withErrors("IMEI Not Found In This Location");
                }
            }

            // $account = AccountMastersAccountsDetail::Where('account_master_id', $request->account_id)->Where('voucher_type', "sale")->Where('voucher_no', $sales_invoice->invoice_no)->first();
            $account->account_master_id = $request->account_id;
            if ($previous_net_total > $current_net_total) {
                $diff = $previous_net_total - $current_net_total;
                $diff_calculate = (float)$account->debit - (float)$diff;
                $account->debit =  number_format((float)$diff_calculate, $request->currency_decimal, '.', '');
            }
            elseif ($previous_net_total < $current_net_total) {
                $diff = $current_net_total - $previous_net_total;
                $diff_calculate = (float)$account->debit + (float)$diff;
                $account->debit =  number_format((float)$diff_calculate, $request->currency_decimal, '.', '');
            }

            $account->save();

            //for tax account
            $tax_account = AccountMastersAccountsDetail::Where('account_master_id', $request->tax_account_id)->Where('voucher_type', "sale")->Where('voucher_no', $sales_invoice->invoice_no)->first();
            $tax_account->account_master_id = $request->tax_account_id;
            $tax_account->credit = $request->vat_amount;
            $tax_account->debit = 0;
            $tax_account->date = $request->date;
            $tax_account->voucher_type = "sale";
            $tax_account->voucher_no = $request->invoice_no;
            $tax_account->save();


            DB::commit();
            Session::flash('alert-success','Sales Invoice updated successfully');
            return redirect('/transactions/sales-invoices');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\SaleInvoice  $saleInvoice
     * @return \Illuminate\Http\Response
     */
    public function destroy(SaleInvoice $saleInvoice)
    {
        //
    }

    public function view($id)
    {
        $user = Auth::user();
        if (!$user->can('view-print-sales-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $sales_invoice = SaleInvoice::withTrashed()->with('items_cancelled.name')->where('id', $id)->first();
        // dd($sales_invoice);
        $buyer = AccountMaster::Where('id', $sales_invoice->account_id)->first();
        $company = Company::where('location_id', $location_id)->first();
        $warehouses = WareHouse::all();
        $salesmen = StaffAndSalesman::where('job_designation', 'salesman')->get();

        $account_masters = [];
        $account_master_tracks = AccountMasterTrack::Where('user_id', auth()->user()->id)->get();
        foreach ($account_master_tracks as $key => $account_master_track){
            $account_masters[] = AccountMaster::Where('location_id', $location_id)->Where('id', $account_master_track->account_master_id)->first();
        }

        $item_names = ItemName::all();
        $currency = Currency::Where('location_id', $location_id)->first();
        $terms_and_conditions_sales_invoice = TermsAndConditionsSalesInvoice::where('location_id', $location_id)->first();
        // dd($sales_invoice->delivered_order_number);
        $tax_account = AccountMaster::where('id', $sales_invoice->tax_account_id)->first();
        return view('user.print.sales_invoice.index', compact('buyer','currency', 'sales_invoice', 'company', 'warehouses', 'salesmen', 'tax_account', 'account_masters','item_names', 'terms_and_conditions_sales_invoice'));
    }

    public function softdelete($id)
    {
        $user = Auth::user();
        if (!$user->can('cancel-sales-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $sales_invoice = SaleInvoice::Find($id);
        $net_total = $sales_invoice->net_total;
        $account_id = $sales_invoice->account_id;

        $account = AccountMastersAccountsDetail::Where('account_master_id', $account_id)->Where('voucher_type', "sale")->Where('voucher_no', $sales_invoice->invoice_no)->first();

        // $debit = $account->debit;
        // $account->debit = (float)$debit - (float)$net_total;
        $account->delete();

        $sales_invoice->delete();

        $sales_invoice_items = SaleInvoiceItem::Where('sales_invoice_id', $id)->get();
        foreach ($sales_invoice_items as $item) {
            $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $item->imei_serial_no)->first();
            if ($purchase_sales_transfer_stock_track) {
                $purchase_sales_transfer_stock_track->quantity = $purchase_sales_transfer_stock_track->quantity + $item->quantity;
                $purchase_sales_transfer_stock_track->save();
            }else{
                return redirect()->back()->withErrors("IMEI Not Found");
            }

            $item->delete();
        }




        Session::flash('alert-success','Sales Invoice Cancelled successfully');
        return redirect('/transactions/sales-invoices');
    }

    public function signature($id){
        $user = Auth::user();
        if (!$user->can('sales-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $sales_invoice = SaleInvoice::Find($id);
        return view('user.transactions.sales_invoices.signature', compact('sales_invoice'));
    }

    public function signatureUpload(Request $request, $id){
        $sales_invoice = SaleInvoice::Find($id);
        $folderPath = public_path('/user_files/img/client_signs/');
        $image_parts = explode(";base64,", $request->signed);
        $image_type_aux = explode("image/", $image_parts[0]);
        $image_type = $image_type_aux[1];
        $image_base64 = base64_decode($image_parts[1]);
        $file_name = 'saleinvoice-' . $id . '.'.$image_type;
        $file = $folderPath . $file_name;
        file_put_contents($file, $image_base64);

        $sales_invoice->is_signed = 1;
        $sales_invoice->signature_path = $file_name;
        $sales_invoice->save();

        Session::flash('alert-success','Sales Invoice Signature Uploaded successfully');
        return redirect('/transactions/sales-invoices');
    }
}
